import { Component, OnInit } from '@angular/core';
import { Observable } from 'rxjs';
import { HttpErrorResponse, HttpResponse } from '@angular/common/http';
import { FormBuilder, FormControl } from '@angular/forms';
import { retry } from 'rxjs/operators';
import { EmployeeService } from './employee.service';
import { Employee } from './employee';
import { CompanyInfo } from './company';

@Component({
	selector: 'app-employee',
	templateUrl: './employee.component.html'
})
export class EmployeeComponent implements OnInit {
	obsTextMsg: Observable<string>;
	obsEmployee: Observable<Employee[]>;
	employee: Employee[] = [];
	favEmployee: Employee[] = [];
	favEmp = {} as CompanyInfo;
	CompanyInfo = {} as CompanyInfo;

	names = [
		{ name: 'Ramesh' },
		{ name: 'Shantosh' },
		{ name: 'Ashutosh' },
		{ name: 'Shobhit' },
	];

	departments = [
		{ department: 'Angular Team' },
		{ department: 'React Team' },
		{ department: 'Java Team' },
		{ department: 'UI Team' }
	];
	

	constructor(private employeeService: EmployeeService,
		private formBuilder: FormBuilder) {
		this.obsTextMsg = this.employeeService.getTextMsg();
		this.obsEmployee = this.employeeService.getEmployee();
	}

	ngOnInit() {
		this.getTextMsg();
		this.getEmployee();
		this.getFavEmployee();
		this.getCompanyName();
		this.employeeForm = this.formBuilder.group({
			name: '',
			department: ''
		});
	}
	employeeForm = this.formBuilder.group({
		name: [''],
		department: ['']
	});

	getName() {
		return this.employeeForm.get('name') as FormControl;
	}
	getDepartment() {
		return this.employeeForm.get('department') as FormControl;
	}

	onFormSubmit() {
		let name = this.getName().value;
		let department = this.getDepartment().value;
		this.filterEmployee(name, department);
	}

	getTextMsg() {
		this.obsTextMsg = this.employeeService.getTextMsg();
		this.employeeService.getTextMsg().subscribe(
			(msg: string) => console.log(msg)
		);
	}
	getEmployee() {
		this.obsEmployee = this.employeeService.getEmployee();
	}
	filterEmployee(name: string, department: string) {
		this.employeeService.filterEmployee(name, department)
			.subscribe((data: Employee[]) => this.employee = data);
	}
	getFavEmployee() {
		this.employeeService.getFavEmployee().subscribe(
			(data: any) => this.favEmployee = data['enployee']
		);
	}
	getCompanyName() {
		this.employeeService.getCompanyName().pipe(
			retry(2)
		).subscribe(
			(data: CompanyInfo) => {
				this.favEmp = data;
				console.log(this.favEmp.employee);
			},
			(err: HttpErrorResponse) => {
				if (err.error instanceof Error) {
					//A client-side or network error occurred.				 
					console.log('An error occurred:', err.error.message);
				} else {
					//Backend returns unsuccessful response codes such as 404, 500 etc.				 
					console.log('Backend returned status code: ', err.status);
					console.log('Response body:', err.error);
				}
			}
		);
	}
}